@@ -1,6 +1,6 @@
 /*
  * sysstat: System performance tools for Linux
- * (C) 1999-2014 by Sebastien Godard (sysstat <at> orange.fr)
+ * (C) 1999-2016 by Sebastien Godard (sysstat <at> orange.fr)
  */
 
 #ifndef _COMMON_H
@@ -11,6 +11,13 @@
 
 #include <time.h>
 #include <sched.h>	/* For __CPU_SETSIZE */
+#include <limits.h>
+
+#ifdef HAVE_SYS_SYSMACROS_H
+/* Needed on some non-glibc environments */
+#include <sys/sysmacros.h>
+#endif
+
 #include "rd_stats.h"
 
 /*
@@ -22,20 +29,22 @@
 #define FALSE	0
 #define TRUE	1
 
+#define PLAIN_OUTPUT	0
+
 #define DISP_HDR	1
 
 /* Number of seconds per day */
 #define SEC_PER_DAY	3600 * 24
 
 /* Maximum number of CPUs */
-#if defined(__CPU_SETSIZE) && __CPU_SETSIZE > 2048
+#if defined(__CPU_SETSIZE) && __CPU_SETSIZE > 8192
 #define NR_CPUS		__CPU_SETSIZE
 #else
-#define NR_CPUS		2048
+#define NR_CPUS		8192
 #endif
 
 /* Maximum number of interrupts */
-#define NR_IRQS			1024
+#define NR_IRQS		1024
 
 /* Size of /proc/interrupts line, CPU data excluded */
 #define INTERRUPTS_LINE	128
@@ -44,6 +53,7 @@
 #define K_ISO	"ISO"
 #define K_ALL	"ALL"
 #define K_UTC	"UTC"
+#define K_JSON	"JSON"
 
 /* Files */
 #define STAT			"/proc/stat"
@@ -65,11 +75,11 @@
 #define SYSFS_BMAXPOWER		"bMaxPower"
 #define SYSFS_MANUFACTURER	"manufacturer"
 #define SYSFS_PRODUCT		"product"
+#define SYSFS_FCHOST		"/sys/class/fc_host"
 
 #define MAX_FILE_LEN		256
 #define MAX_PF_NAME		1024
-#define DEFAULT_DEVMAP_MAJOR	253
-#define MAX_NAME_LEN		72
+#define MAX_NAME_LEN		128
 
 #define IGNORE_VIRTUAL_DEVICES	FALSE
 #define ACCEPT_VIRTUAL_DEVICES	TRUE
@@ -77,6 +87,11 @@
 /* Environment variables */
 #define ENV_TIME_FMT		"S_TIME_FORMAT"
 #define ENV_TIME_DEFTM		"S_TIME_DEF_TIME"
+#define ENV_COLORS		"S_COLORS"
+#define ENV_COLORS_SGR		"S_COLORS_SGR"
+
+#define C_NEVER			"never"
+#define C_ALWAYS		"always"
 
 #define DIGITS			"0123456789"
 
@@ -96,9 +111,15 @@
 				         		exit(4);				 \
 				      		}						 \
 				      		/* If the ptr was null, then it's a malloc() */	 \
-   				      		if (!_p_)					 \
-      				         		memset(S, 0, (SIZE));			 \
+						if (!_p_) {					 \
+							memset(S, 0, (SIZE));			 \
+						}						 \
 				   	}							 \
+					if (!S) {						 \
+						/* Should never happen */			 \
+						fprintf(stderr, "srealloc\n");		 	 \
+						exit(4);					 \
+					}							 \
 				} while (0)
 
 /*
@@ -106,9 +127,12 @@
  *
  * NB: Define SP_VALUE() to normalize to %;
  * HZ is 1024 on IA64 and % should be normalized to 100.
+ * SP_VALUE_100() will not output value bigger than 100; this is needed for some
+ * corner cases, should be used with care.
  */
-#define S_VALUE(m,n,p)	(((double) ((n) - (m))) / (p) * HZ)
-#define SP_VALUE(m,n,p)	(((double) ((n) - (m))) / (p) * 100)
+#define S_VALUE(m,n,p)		(((double) ((n) - (m))) / (p) * HZ)
+#define SP_VALUE(m,n,p)		(((double) ((n) - (m))) / (p) * 100)
+#define SP_VALUE_100(m,n,p)	MINIMUM((((double) ((n) - (m))) / (p) * 100), 100.0)
 
 /*
  * Under very special circumstances, STDOUT may become unavailable.
@@ -123,11 +147,7 @@
 
 #define MINIMUM(a,b)	((a) < (b) ? (a) : (b))
 
-#ifdef DEBUG
 #define PANIC(m)	sysstat_panic(__FUNCTION__, m)
-#else
-#define PANIC(m)
-#endif
 
 /* Number of ticks per second */
 #define HZ		hz
@@ -146,6 +166,32 @@ extern unsigned int kb_shift;
 /* Type of persistent device names used in sar and iostat */
 extern char persistent_name_type[MAX_FILE_LEN];
 
+/*
+ ***************************************************************************
+ * Colors definitions
+ ***************************************************************************
+ */
+
+#define C_LIGHT_RED	"\e[31;22m"
+#define C_BOLD_RED	"\e[31;1m"
+#define C_LIGHT_GREEN	"\e[32;22m"
+#define C_LIGHT_YELLOW	"\e[33;22m"
+#define C_BOLD_YELLOW	"\e[33;1m"
+#define C_BOLD_BLUE	"\e[34;1m"
+#define C_LIGHT_CYAN	"\e[36;22m"
+#define C_NORMAL	"\e[0m"
+
+#define PERCENT_LIMIT_HIGH	75.0
+#define PERCENT_LIMIT_LOW	50.0
+
+#define MAX_SGR_LEN	16
+
+#define IS_INT		0
+#define IS_STR		1
+#define IS_RESTART	2
+#define IS_COMMENT	3
+#define IS_ZERO		4
+
 /*
  ***************************************************************************
  * Structures definitions
@@ -166,56 +212,72 @@ struct ext_disk_stats {
  ***************************************************************************
  */
 
-extern void
-	compute_ext_disk_stats(struct stats_disk *, struct stats_disk *,
-			       unsigned long long, struct ext_disk_stats *);
-extern int
-	count_bits(void *, int);
-extern int
-	count_csvalues(int, char **);
-extern char *
-	device_name(char *);
-extern void
-	get_HZ(void);
-extern unsigned int
-	get_devmap_major(void);
-extern unsigned long long
-	get_interval(unsigned long long, unsigned long long);
-extern void
-	get_kb_shift(void);
-extern time_t
-	get_localtime(struct tm *, int);
-extern time_t
-	get_time(struct tm *, int);
-unsigned long long
-	get_per_cpu_interval(struct stats_cpu *, struct stats_cpu *);
-extern char *
-	get_persistent_name_from_pretty(char *);
-extern char *
-	get_persistent_type_dir(char *);
-extern char *
-	get_pretty_name_from_persistent(char *);
-extern int
-	get_sysfs_dev_nr(int);
-extern int
-	get_win_height(void);
-extern void
-	init_nls(void);
-extern int
-	is_device(char *, int);
-extern double
-	ll_s_value(unsigned long long, unsigned long long, unsigned long long);
-extern double
-	ll_sp_value(unsigned long long, unsigned long long, unsigned long long);
-extern int
-	print_gal_header(struct tm *, char *, char *, char *, char *, int);
-extern void
-	print_version(void);
-extern char *
-	strtolower(char *);
-#ifdef DEBUG
-extern void
-	sysstat_panic(const char *, int);
-#endif
+void compute_ext_disk_stats
+	(struct stats_disk *, struct stats_disk *, unsigned long long,
+	 struct ext_disk_stats *);
+int count_bits
+	(void *, int);
+int count_csvalues
+	(int, char **);
+void cprintf_f
+	(int, int, int, ...);
+void cprintf_in
+	(int, char *, char *, int);
+void cprintf_pc
+	(int, int, int, ...);
+void cprintf_s
+	(int, char *, char *);
+void cprintf_u64
+	(int, int, ...);
+void cprintf_x
+	(int, int, ...);
+char *device_name
+	(char *);
+void get_HZ
+	(void);
+unsigned int get_devmap_major
+	(void);
+unsigned long long get_interval
+	(unsigned long long, unsigned long long);
+void get_kb_shift
+	(void);
+time_t get_localtime
+	(struct tm *, int);
+time_t get_time
+	(struct tm *, int);
+unsigned long long get_per_cpu_interval
+	(struct stats_cpu *, struct stats_cpu *);
+char *get_persistent_name_from_pretty
+	(char *);
+char *get_persistent_type_dir
+	(char *);
+char *get_pretty_name_from_persistent
+	(char *);
+int get_sysfs_dev_nr
+	(int);
+int get_win_height
+	(void);
+void init_colors
+	(void);
+void init_nls
+	(void);
+int is_device
+	(char *, int);
+double ll_sp_value
+	(unsigned long long, unsigned long long, unsigned long long);
+int is_iso_time_fmt
+	(void);
+int print_gal_header
+	(struct tm *, char *, char *, char *, char *, int, int);
+void print_version
+	(void);
+char *strtolower
+	(char *);
+void sysstat_panic
+	(const char *, int);
+void xprintf
+	(int, const char *, ...);
+void xprintf0
+	(int, const char *, ...);
 
 #endif  /* _COMMON_H */
