@@ -1,6 +1,6 @@
 /*
  * iostat: report CPU and I/O statistics
- * (C) 1998-2014 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1998-2016 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -15,7 +15,7 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc., *
- * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                   *
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA              *
  ***************************************************************************
  */
 
@@ -88,14 +88,14 @@ void usage(char *progname)
 #ifdef DEBUG
 	fprintf(stderr, _("Options are:\n"
 			  "[ -c ] [ -d ] [ -h ] [ -k | -m ] [ -N ] [ -t ] [ -V ] [ -x ] [ -y ] [ -z ]\n"
-			  "[ -j { ID | LABEL | PATH | UUID | ... } ]\n"
-			  "[ [ -T ] -g <group_name> ] [ -p [ <device> [,...] | ALL ] ]\n"
+			  "[ -j { ID | LABEL | PATH | UUID | ... } ] [ -o JSON ]\n"
+			  "[ [ -H ] -g <group_name> ] [ -p [ <device> [,...] | ALL ] ]\n"
 			  "[ <device> [...] | ALL ] [ --debuginfo ]\n"));
 #else
 	fprintf(stderr, _("Options are:\n"
 			  "[ -c ] [ -d ] [ -h ] [ -k | -m ] [ -N ] [ -t ] [ -V ] [ -x ] [ -y ] [ -z ]\n"
-			  "[ -j { ID | LABEL | PATH | UUID | ... } ]\n"
-			  "[ [ -T ] -g <group_name> ] [ -p [ <device> [,...] | ALL ] ]\n"
+			  "[ -j { ID | LABEL | PATH | UUID | ... } ] [ -o JSON ]\n"
+			  "[ [ -H ] -g <group_name> ] [ -p [ <device> [,...] | ALL ] ]\n"
 			  "[ <device> [...] | ALL ]\n"));
 #endif
 	exit(1);
@@ -110,13 +110,11 @@ void usage(char *progname)
  */
 void set_disk_output_unit(void)
 {
-	char *e;
-
 	if (DISPLAY_KILOBYTES(flags) || DISPLAY_MEGABYTES(flags))
 		return;
 
 	/* Check POSIXLY_CORRECT environment variable */
-	if ((e = getenv(ENV_POSIXLY_CORRECT)) == NULL) {
+	if (getenv(ENV_POSIXLY_CORRECT) == NULL) {
 		/* Variable not set: Unit is kB/s and not blocks/s */
 		flags |= I_D_KILOBYTES;
 	}
@@ -365,7 +363,8 @@ void presave_device_list(void)
 
 		/* Now save devices and group names in the io_hdr_stats structures */
 		for (i = 0; (i < dlist_idx) && (i < iodev_nr); i++, shi++, sdli++) {
-			strcpy(shi->name, sdli->dev_name);
+			strncpy(shi->name, sdli->dev_name, MAX_NAME_LEN);
+			shi->name[MAX_NAME_LEN - 1] = '\0';
 			shi->used = TRUE;
 			if (shi->name[0] == ' ') {
 				/* Current device name is in fact the name of a group */
@@ -384,7 +383,8 @@ void presave_device_list(void)
 		 * included in that group.
 		 */
 		shi += iodev_nr - 1;
-		strcpy(shi->name, group_name);
+		strncpy(shi->name, group_name, MAX_NAME_LEN);
+		shi->name[MAX_NAME_LEN - 1] = '\0';
 		shi->used = TRUE;
 		shi->status = DISK_GROUP;
 	}
@@ -450,7 +450,8 @@ void save_stats(char *name, int curr, void *st_io, int iodev_nr,
 			if (!st_hdr_iodev_i->used) {
 				/* Unused entry found... */
 				st_hdr_iodev_i->used = TRUE; /* Indicate it is now used */
-				strcpy(st_hdr_iodev_i->name, name);
+				strncpy(st_hdr_iodev_i->name, name, MAX_NAME_LEN - 1);
+				st_hdr_iodev_i->name[MAX_NAME_LEN - 1] = '\0';
 				st_iodev_i = st_iodev[!curr] + i;
 				memset(st_iodev_i, 0, IO_STATS_SIZE);
 				break;
@@ -740,7 +741,8 @@ void read_diskstats_stat(int curr)
 				 * (if different from "nodev") works around known issues
 				 * with EMC PowerPath.
 				 */
-				strncpy(dev_name, ioc_dname, MAX_NAME_LEN);
+				strncpy(dev_name, ioc_dname, MAX_NAME_LEN - 1);
+				dev_name[MAX_NAME_LEN - 1] = '\0';
 			}
 		}
 
@@ -751,7 +753,8 @@ void read_diskstats_stat(int curr)
 			 */
 			dm_name = transform_devmapname(major, minor);
 			if (dm_name) {
-				strncpy(dev_name, dm_name, MAX_NAME_LEN);
+				strncpy(dev_name, dm_name, MAX_NAME_LEN - 1);
+				dev_name[MAX_NAME_LEN - 1] = '\0';
 			}
 		}
 
@@ -814,55 +817,145 @@ void compute_device_groups_stats(int curr)
 
 /*
  ***************************************************************************
- * Display CPU utilization.
+ * Write current sample's timestamp, either in plain or JSON format.
+ *
+ * IN:
+ * @tab		Number of tabs to print.
+ * @rectime	Current date and time.
+ ***************************************************************************
+ */
+void write_sample_timestamp(int tab, struct tm *rectime)
+{
+	if (DISPLAY_ISO(flags)) {
+		strftime(timestamp, sizeof(timestamp), "%FT%T%z", rectime);
+	}
+	else {
+		strftime(timestamp, sizeof(timestamp), "%x %X", rectime);
+	}
+	if (DISPLAY_JSON_OUTPUT(flags)) {
+		xprintf(tab, "\"timestamp\": \"%s\",", timestamp);
+	}
+	else {
+		printf("%s\n", timestamp);
+	}
+}
+
+/*
+ ***************************************************************************
+ * Display CPU utilization in plain format.
  *
  * IN:
  * @curr	Index in array for current sample statistics.
  * @itv		Interval of time.
  ***************************************************************************
  */
-void write_cpu_stat(int curr, unsigned long long itv)
+void write_plain_cpu_stat(int curr, unsigned long long itv)
 {
 	printf("avg-cpu:  %%user   %%nice %%system %%iowait  %%steal   %%idle\n");
 
-	printf("         %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f\n\n",
-	       ll_sp_value(st_cpu[!curr]->cpu_user,   st_cpu[curr]->cpu_user,   itv),
-	       ll_sp_value(st_cpu[!curr]->cpu_nice,   st_cpu[curr]->cpu_nice,   itv),
-		/*
-		 * Time spent in system mode also includes time spent servicing
-		 * hard and soft interrupts.
-		 */
-	       ll_sp_value(st_cpu[!curr]->cpu_sys + st_cpu[!curr]->cpu_softirq +
-			   st_cpu[!curr]->cpu_hardirq,
-			   st_cpu[curr]->cpu_sys + st_cpu[curr]->cpu_softirq +
-			   st_cpu[curr]->cpu_hardirq, itv),
-	       ll_sp_value(st_cpu[!curr]->cpu_iowait, st_cpu[curr]->cpu_iowait, itv),
-	       ll_sp_value(st_cpu[!curr]->cpu_steal,  st_cpu[curr]->cpu_steal,  itv),
-	       (st_cpu[curr]->cpu_idle < st_cpu[!curr]->cpu_idle) ?
-	       0.0 :
-	       ll_sp_value(st_cpu[!curr]->cpu_idle,   st_cpu[curr]->cpu_idle,   itv));
+	printf("       ");
+	cprintf_pc(6, 7, 2,
+		   ll_sp_value(st_cpu[!curr]->cpu_user,   st_cpu[curr]->cpu_user,   itv),
+		   ll_sp_value(st_cpu[!curr]->cpu_nice,   st_cpu[curr]->cpu_nice,   itv),
+		   /*
+		    * Time spent in system mode also includes time spent servicing
+		    * hard and soft interrupts.
+		    */
+		   ll_sp_value(st_cpu[!curr]->cpu_sys + st_cpu[!curr]->cpu_softirq +
+			       st_cpu[!curr]->cpu_hardirq,
+			       st_cpu[curr]->cpu_sys + st_cpu[curr]->cpu_softirq +
+			       st_cpu[curr]->cpu_hardirq, itv),
+		   ll_sp_value(st_cpu[!curr]->cpu_iowait, st_cpu[curr]->cpu_iowait, itv),
+		   ll_sp_value(st_cpu[!curr]->cpu_steal,  st_cpu[curr]->cpu_steal,  itv),
+		   (st_cpu[curr]->cpu_idle < st_cpu[!curr]->cpu_idle) ?
+		   0.0 :
+		   ll_sp_value(st_cpu[!curr]->cpu_idle,   st_cpu[curr]->cpu_idle,   itv));
+
+	printf("\n\n");
 }
 
 /*
  ***************************************************************************
- * Display disk stats header.
+ * Display CPU utilization in JSON format.
+ *
+ * IN:
+ * @tab		Number of tabs to print.
+ * @curr	Index in array for current sample statistics.
+ * @itv		Interval of time.
+ ***************************************************************************
+ */
+void write_json_cpu_stat(int tab, int curr, unsigned long long itv)
+{
+	xprintf0(tab, "\"avg-cpu\":  {\"user\": %.2f, \"nice\": %.2f, \"system\": %.2f,"
+		      " \"iowait\": %.2f, \"steal\": %.2f, \"idle\": %.2f}",
+		 ll_sp_value(st_cpu[!curr]->cpu_user,   st_cpu[curr]->cpu_user,   itv),
+		 ll_sp_value(st_cpu[!curr]->cpu_nice,   st_cpu[curr]->cpu_nice,   itv),
+		 /*
+		  * Time spent in system mode also includes time spent servicing
+		  * hard and soft interrupts.
+		  */
+		 ll_sp_value(st_cpu[!curr]->cpu_sys + st_cpu[!curr]->cpu_softirq +
+			     st_cpu[!curr]->cpu_hardirq,
+			     st_cpu[curr]->cpu_sys + st_cpu[curr]->cpu_softirq +
+			     st_cpu[curr]->cpu_hardirq, itv),
+		 ll_sp_value(st_cpu[!curr]->cpu_iowait, st_cpu[curr]->cpu_iowait, itv),
+		 ll_sp_value(st_cpu[!curr]->cpu_steal,  st_cpu[curr]->cpu_steal,  itv),
+		 (st_cpu[curr]->cpu_idle < st_cpu[!curr]->cpu_idle) ?
+		 0.0 :
+		 ll_sp_value(st_cpu[!curr]->cpu_idle,   st_cpu[curr]->cpu_idle,   itv));
+}
+
+/*
+ ***************************************************************************
+ * Display CPU utilization in plain or JSON format.
+ *
+ * IN:
+ * @curr	Index in array for current sample statistics.
+ * @itv		Interval of time.
+ * @tab		Number of tabs to print (JSON format only).
+ ***************************************************************************
+ */
+void write_cpu_stat(int curr, unsigned long long itv, int tab)
+{
+	if (DISPLAY_JSON_OUTPUT(flags)) {
+		write_json_cpu_stat(tab, curr, itv);
+	}
+	else {
+		write_plain_cpu_stat(curr, itv);
+	}
+}
+
+/*
+ ***************************************************************************
+ * Display disk stats header in plain or JSON format.
  *
  * OUT:
  * @fctr	Conversion factor.
+ * @tab		Number of tabs to print (JSON format only).
  ***************************************************************************
  */
-void write_disk_stat_header(int *fctr)
+void write_disk_stat_header(int *fctr, int *tab)
 {
+	if (DISPLAY_KILOBYTES(flags)) {
+		*fctr = 2;
+	}
+	else if (DISPLAY_MEGABYTES(flags)) {
+		*fctr = 2048;
+	}
+
+	if (DISPLAY_JSON_OUTPUT(flags)) {
+		xprintf((*tab)++, "\"disk\": [");
+		return;
+	}
+
 	if (DISPLAY_EXTENDED(flags)) {
 		/* Extended stats */
 		printf("Device:         rrqm/s   wrqm/s     r/s     w/s");
 		if (DISPLAY_MEGABYTES(flags)) {
 			printf("    rMB/s    wMB/s");
-			*fctr = 2048;
 		}
 		else if (DISPLAY_KILOBYTES(flags)) {
 			printf("    rkB/s    wkB/s");
-			*fctr = 2;
 		}
 		else {
 			printf("   rsec/s   wsec/s");
@@ -874,11 +967,9 @@ void write_disk_stat_header(int *fctr)
 		printf("Device:            tps");
 		if (DISPLAY_KILOBYTES(flags)) {
 			printf("    kB_read/s    kB_wrtn/s    kB_read    kB_wrtn\n");
-			*fctr = 2;
 		}
 		else if (DISPLAY_MEGABYTES(flags)) {
 			printf("    MB_read/s    MB_wrtn/s    MB_read    MB_wrtn\n");
-			*fctr = 2048;
 		}
 		else {
 			printf("   Blk_read/s   Blk_wrtn/s   Blk_read   Blk_wrtn\n");
@@ -888,20 +979,123 @@ void write_disk_stat_header(int *fctr)
 
 /*
  ***************************************************************************
- * Display extended stats, read from /proc/{diskstats,partitions} or /sys.
+ * Display extended stats, read from /proc/{diskstats,partitions} or /sys,
+ * in plain format.
  *
  * IN:
- * @curr	Index in array for current sample statistics.
  * @itv		Interval of time.
  * @fctr	Conversion factor.
  * @shi		Structures describing the devices and partitions.
  * @ioi		Current sample statistics.
  * @ioj		Previous sample statistics.
+ * @devname	Current device name.
+ * @xds		Extended stats for current device.
+ * @r_await	r_await metric value.
+ * @w_await	w_await metric value.
  ***************************************************************************
  */
-void write_ext_stat(int curr, unsigned long long itv, int fctr,
+void write_plain_ext_stat(unsigned long long itv, int fctr,
+			  struct io_hdr_stats *shi, struct io_stats *ioi,
+			  struct io_stats *ioj, char *devname, struct ext_disk_stats *xds,
+			  double r_await, double w_await)
+{
+	if (DISPLAY_HUMAN_READ(flags)) {
+		cprintf_in(IS_STR, "%s\n", devname, 0);
+		printf("%13s", "");
+	}
+	else {
+		cprintf_in(IS_STR, "%-13s", devname, 0);
+	}
+
+	/*       rrq/s wrq/s   r/s   w/s  rsec  wsec  rqsz  qusz await r_await w_await svctm %util */
+	cprintf_f(2, 8, 2,
+		  S_VALUE(ioj->rd_merges, ioi->rd_merges, itv),
+		  S_VALUE(ioj->wr_merges, ioi->wr_merges, itv));
+	cprintf_f(2, 7, 2,
+		  S_VALUE(ioj->rd_ios, ioi->rd_ios, itv),
+		  S_VALUE(ioj->wr_ios, ioi->wr_ios, itv));
+	cprintf_f(4, 8, 2,
+		  S_VALUE(ioj->rd_sectors, ioi->rd_sectors, itv) / fctr,
+		  S_VALUE(ioj->wr_sectors, ioi->wr_sectors, itv) / fctr,
+		  xds->arqsz,
+		  S_VALUE(ioj->rq_ticks, ioi->rq_ticks, itv) / 1000.0);
+	cprintf_f(3, 7, 2, xds->await, r_await, w_await);
+	/* The ticks output is biased to output 1000 ticks per second */
+	cprintf_f(1, 6, 2, xds->svctm);
+	/*
+	 * Again: Ticks in milliseconds.
+	 * In the case of a device group (option -g), shi->used is the number of
+	 * devices in the group. Else shi->used equals 1.
+	 */
+	cprintf_pc(1, 6, 2,
+		   shi->used ? xds->util / 10.0 / (double) shi->used
+		             : xds->util / 10.0);	/* shi->used should never be zero here */
+	printf("\n");
+}
+
+/*
+ ***************************************************************************
+ * Display extended stats, read from /proc/{diskstats,partitions} or /sys,
+ * in JSON format.
+ *
+ * IN:
+ * @tab		Number of tabs to print.
+ * @itv		Interval of time.
+ * @fctr	Conversion factor.
+ * @shi		Structures describing the devices and partitions.
+ * @ioi		Current sample statistics.
+ * @ioj		Previous sample statistics.
+ * @devname	Current device name.
+ * @xds		Extended stats for current device.
+ * @r_await	r_await metric value.
+ * @w_await	w_await metric value.
+ ***************************************************************************
+ */
+void write_json_ext_stat(int tab, unsigned long long itv, int fctr,
 		    struct io_hdr_stats *shi, struct io_stats *ioi,
-		    struct io_stats *ioj)
+		    struct io_stats *ioj, char *devname, struct ext_disk_stats *xds,
+		    double r_await, double w_await)
+{
+	xprintf0(tab,
+		 "{\"disk_device\": \"%s\", \"rrqm\": %.2f, \"wrqm\": %.2f, "
+		 "\"r\": %.2f, \"w\": %.2f, \"rkB\": %.2f, \"wkB\": %.2f, "
+		 "\"avgrq-sz\": %.2f, \"avgqu-sz\": %.2f, "
+		 "\"await\": %.2f, \"r_await\": %.2f, \"w_await\": %.2f, "
+		 "\"svctm\": %.2f, \"util\": %.2f}",
+		 devname,
+		 S_VALUE(ioj->rd_merges, ioi->rd_merges, itv),
+		 S_VALUE(ioj->wr_merges, ioi->wr_merges, itv),
+		 S_VALUE(ioj->rd_ios, ioi->rd_ios, itv),
+		 S_VALUE(ioj->wr_ios, ioi->wr_ios, itv),
+		 S_VALUE(ioj->rd_sectors, ioi->rd_sectors, itv) / fctr,
+		 S_VALUE(ioj->wr_sectors, ioi->wr_sectors, itv) / fctr,
+		 xds->arqsz,
+		 S_VALUE(ioj->rq_ticks, ioi->rq_ticks, itv) / 1000.0,
+		 xds->await,
+		 r_await,
+		 w_await,
+		 xds->svctm,
+		 shi->used ? xds->util / 10.0 / (double) shi->used
+			   : xds->util / 10.0);	/* shi->used should never be zero here */
+}
+
+/*
+ ***************************************************************************
+ * Display extended stats, read from /proc/{diskstats,partitions} or /sys,
+ * in plain or JSON format.
+ *
+ * IN:
+ * @itv		Interval of time.
+ * @fctr	Conversion factor.
+ * @shi		Structures describing the devices and partitions.
+ * @ioi		Current sample statistics.
+ * @ioj		Previous sample statistics.
+ * @tab		Number of tabs to print (JSON output only).
+ ***************************************************************************
+ */
+void write_ext_stat(unsigned long long itv, int fctr,
+		    struct io_hdr_stats *shi, struct io_stats *ioi,
+		    struct io_stats *ioj, int tab)
 {
 	char *devname = NULL;
 	struct stats_disk sdc, sdp;
@@ -942,60 +1136,112 @@ void write_ext_stat(int curr, unsigned long long itv, int fctr,
 		  (ioi->wr_ticks - ioj->wr_ticks) /
 		  ((double) (ioi->wr_ios - ioj->wr_ios)) : 0.0;
 
-	/* Print device name */
+	/* Get device name */
 	if (DISPLAY_PERSIST_NAME_I(flags)) {
 		devname = get_persistent_name_from_pretty(shi->name);
 	}
 	if (!devname) {
 		devname = shi->name;
 	}
-	if (DISPLAY_HUMAN_READ(flags)) {
-		printf("%s\n%13s", devname, "");
+
+	if (DISPLAY_JSON_OUTPUT(flags)) {
+		write_json_ext_stat(tab, itv, fctr, shi, ioi, ioj, devname, &xds,
+				    r_await, w_await);
 	}
 	else {
-		printf("%-13s", devname);
+		write_plain_ext_stat(itv, fctr, shi, ioi, ioj, devname, &xds,
+				     r_await, w_await);
 	}
-
-	/*       rrq/s wrq/s   r/s   w/s  rsec  wsec  rqsz  qusz await r_await w_await svctm %util */
-	printf(" %8.2f %8.2f %7.2f %7.2f %8.2f %8.2f %8.2f %8.2f %7.2f %7.2f %7.2f %6.2f %6.2f\n",
-	       S_VALUE(ioj->rd_merges, ioi->rd_merges, itv),
-	       S_VALUE(ioj->wr_merges, ioi->wr_merges, itv),
-	       S_VALUE(ioj->rd_ios, ioi->rd_ios, itv),
-	       S_VALUE(ioj->wr_ios, ioi->wr_ios, itv),
-	       ll_s_value(ioj->rd_sectors, ioi->rd_sectors, itv) / fctr,
-	       ll_s_value(ioj->wr_sectors, ioi->wr_sectors, itv) / fctr,
-	       xds.arqsz,
-	       S_VALUE(ioj->rq_ticks, ioi->rq_ticks, itv) / 1000.0,
-	       xds.await,
-	       r_await,
-	       w_await,
-	       /* The ticks output is biased to output 1000 ticks per second */
-	       xds.svctm,
-	       /*
-	        * Again: Ticks in milliseconds.
-		* In the case of a device group (option -g), shi->used is the number of
-		* devices in the group. Else shi->used equals 1.
-		*/
-	       shi->used ? xds.util / 10.0 / (double) shi->used
-	                 : xds.util / 10.0);	/* shi->used should never be null here */
 }
 
 /*
  ***************************************************************************
- * Write basic stats, read from /proc/diskstats or from sysfs.
+ * Write basic stats, read from /proc/diskstats or from sysfs, in plain
+ * format.
+ *
+ * IN:
+ * @itv		Interval of time.
+ * @fctr	Conversion factor.
+ * @ioi		Current sample statistics.
+ * @ioj		Previous sample statistics.
+ * @devname	Current device name.
+ * @rd_sec	Number of sectors read.
+ * @wr_sec	Number of sectors written.
+ ***************************************************************************
+ */
+void write_plain_basic_stat(unsigned long long itv, int fctr,
+			    struct io_stats *ioi, struct io_stats *ioj,
+			    char *devname, unsigned long long rd_sec,
+			    unsigned long long wr_sec)
+{
+	if (DISPLAY_HUMAN_READ(flags)) {
+		cprintf_in(IS_STR, "%s\n", devname, 0);
+		printf("%13s", "");
+	}
+	else {
+		cprintf_in(IS_STR, "%-13s", devname, 0);
+	}
+	cprintf_f(1, 8, 2,
+		  S_VALUE(ioj->rd_ios + ioj->wr_ios, ioi->rd_ios + ioi->wr_ios, itv));
+	cprintf_f(2, 12, 2,
+		  S_VALUE(ioj->rd_sectors, ioi->rd_sectors, itv) / fctr,
+		  S_VALUE(ioj->wr_sectors, ioi->wr_sectors, itv) / fctr);
+	cprintf_u64(2, 10,
+		    (unsigned long long) rd_sec / fctr,
+		    (unsigned long long) wr_sec / fctr);
+	printf("\n");
+}
+
+/*
+ ***************************************************************************
+ * Write basic stats, read from /proc/diskstats or from sysfs, in JSON
+ * format.
+ *
+ * IN:
+ * @tab		Number of tabs to print.
+ * @itv		Interval of time.
+ * @fctr	Conversion factor.
+ * @ioi		Current sample statistics.
+ * @ioj		Previous sample statistics.
+ * @devname	Current device name.
+ * @rd_sec	Number of sectors read.
+ * @wr_sec	Number of sectors written.
+ ***************************************************************************
+ */
+void write_json_basic_stat(int tab, unsigned long long itv, int fctr,
+			   struct io_stats *ioi, struct io_stats *ioj,
+			   char *devname, unsigned long long rd_sec,
+			   unsigned long long wr_sec)
+{
+	xprintf0(tab,
+		 "{\"disk_device\": \"%s\", \"tps\": %.2f, "
+		 "\"kB_read_per_sec\": %.2f, \"kB_wrtn_per_sec\": %.2f, "
+		 "\"kB_read\": %llu, \"kB_wrtn\": %llu}",
+		 devname,
+		 S_VALUE(ioj->rd_ios + ioj->wr_ios, ioi->rd_ios + ioi->wr_ios, itv),
+		 S_VALUE(ioj->rd_sectors, ioi->rd_sectors, itv) / fctr,
+		 S_VALUE(ioj->wr_sectors, ioi->wr_sectors, itv) / fctr,
+		 (unsigned long long) rd_sec / fctr,
+		 (unsigned long long) wr_sec / fctr);
+}
+
+/*
+ ***************************************************************************
+ * Write basic stats, read from /proc/diskstats or from sysfs, in plain or
+ * JSON format.
  *
  * IN:
- * @curr	Index in array for current sample statistics.
  * @itv		Interval of time.
  * @fctr	Conversion factor.
  * @shi		Structures describing the devices and partitions.
  * @ioi		Current sample statistics.
  * @ioj		Previous sample statistics.
+ * @tab		Number of tabs to print (JSON format only).
  ***************************************************************************
  */
-void write_basic_stat(int curr, unsigned long long itv, int fctr,
+void write_basic_stat(unsigned long long itv, int fctr,
 		      struct io_hdr_stats *shi, struct io_stats *ioi,
-		      struct io_stats *ioj)
+		      struct io_stats *ioj, int tab)
 {
 	char *devname = NULL;
 	unsigned long long rd_sec, wr_sec;
@@ -1007,12 +1253,6 @@ void write_basic_stat(int curr, unsigned long long itv, int fctr,
 	if (!devname) {
 		devname = shi->name;
 	}
-	if (DISPLAY_HUMAN_READ(flags)) {
-		printf("%s\n%13s", devname, "");
-	}
-	else {
-		printf("%-13s", devname);
-	}
 
 	/* Print stats coming from /sys or /proc/diskstats */
 	rd_sec = ioi->rd_sectors - ioj->rd_sectors;
@@ -1024,12 +1264,14 @@ void write_basic_stat(int curr, unsigned long long itv, int fctr,
 		wr_sec &= 0xffffffff;
 	}
 
-	printf(" %8.2f %12.2f %12.2f %10llu %10llu\n",
-	       S_VALUE(ioj->rd_ios + ioj->wr_ios, ioi->rd_ios + ioi->wr_ios, itv),
-	       ll_s_value(ioj->rd_sectors, ioi->rd_sectors, itv) / fctr,
-	       ll_s_value(ioj->wr_sectors, ioi->wr_sectors, itv) / fctr,
-	       (unsigned long long) rd_sec / fctr,
-	       (unsigned long long) wr_sec / fctr);
+	if (DISPLAY_JSON_OUTPUT(flags)) {
+		write_json_basic_stat(tab, itv, fctr, ioi, ioj, devname,
+				      rd_sec, wr_sec);
+	}
+	else {
+		write_plain_basic_stat(itv, fctr, ioi, ioj, devname,
+				       rd_sec, wr_sec);
+	}
 }
 
 /*
@@ -1043,7 +1285,7 @@ void write_basic_stat(int curr, unsigned long long itv, int fctr,
  */
 void write_stats(int curr, struct tm *rectime)
 {
-	int dev, i, fctr = 1;
+	int dev, i, fctr = 1, tab = 4, next = FALSE;
 	unsigned long long itv;
 	struct io_hdr_stats *shi;
 	struct io_dlist *st_dev_list_i;
@@ -1051,15 +1293,13 @@ void write_stats(int curr, struct tm *rectime)
 	/* Test stdout */
 	TEST_STDOUT(STDOUT_FILENO);
 
+	if (DISPLAY_JSON_OUTPUT(flags)) {
+		xprintf(tab++, "{");
+	}
+
 	/* Print time stamp */
 	if (DISPLAY_TIMESTAMP(flags)) {
-		if (DISPLAY_ISO(flags)) {
-			strftime(timestamp, sizeof(timestamp), "%FT%T%z", rectime);
-		}
-		else {
-			strftime(timestamp, sizeof(timestamp), "%x %X", rectime);
-		}
-		printf("%s\n", timestamp);
+		write_sample_timestamp(tab, rectime);
 #ifdef DEBUG
 		if (DISPLAY_DEBUG(flags)) {
 			fprintf(stderr, "%s\n", timestamp);
@@ -1093,7 +1333,14 @@ void write_stats(int curr, struct tm *rectime)
 #endif
 
 		/* Display CPU utilization */
-		write_cpu_stat(curr, itv);
+		write_cpu_stat(curr, itv, tab);
+
+		if (DISPLAY_JSON_OUTPUT(flags)) {
+			if (DISPLAY_DISK(flags)) {
+				printf(",");
+			}
+			printf("\n");
+		}
 	}
 
 	if (cpu_nr > 1) {
@@ -1107,7 +1354,7 @@ void write_stats(int curr, struct tm *rectime)
 		shi = st_hdr_iodev;
 
 		/* Display disk stats header */
-		write_disk_stat_header(&fctr);
+		write_disk_stat_header(&fctr, &tab);
 
 		for (i = 0; i < iodev_nr; i++, shi++) {
 			if (shi->used) {
@@ -1173,14 +1420,29 @@ void write_stats(int curr, struct tm *rectime)
 				}
 #endif
 
+				if (DISPLAY_JSON_OUTPUT(flags) && next) {
+					printf(",\n");
+				}
+				next = TRUE;
+
 				if (DISPLAY_EXTENDED(flags)) {
-					write_ext_stat(curr, itv, fctr, shi, ioi, ioj);
+					write_ext_stat(itv, fctr, shi, ioi, ioj, tab);
 				}
 				else {
-					write_basic_stat(curr, itv, fctr, shi, ioi, ioj);
+					write_basic_stat(itv, fctr, shi, ioi, ioj, tab);
 				}
 			}
 		}
+		if (DISPLAY_JSON_OUTPUT(flags)) {
+			printf("\n");
+			xprintf(--tab, "]");
+		}
+	}
+
+	if (DISPLAY_JSON_OUTPUT(flags)) {
+		xprintf0(--tab, "}");
+	}
+	else {
 		printf("\n");
 	}
 }
@@ -1190,7 +1452,7 @@ void write_stats(int curr, struct tm *rectime)
  * Main loop: Read I/O stats from the relevant sources and display them.
  *
  * IN:
- * @count	Number of lines of stats to print.
+ * @count	Number of reports to print.
  * @rectime	Current date and time.
  ***************************************************************************
  */
@@ -1272,12 +1534,22 @@ void rw_io_stat_loop(long int count, struct tm *rectime)
 			skip = 0;
 		}
 
+		if (DISPLAY_JSON_OUTPUT(flags)) {
+			if (count) {
+			printf(",");
+			}
+			printf("\n");
+		}
 		if (count) {
 			curr ^= 1;
 			pause();
 		}
 	}
 	while (count);
+
+	if (DISPLAY_JSON_OUTPUT(flags)) {
+		printf("\t\t\t]\n\t\t}\n\t]\n}}\n");
+	}
 }
 
 /*
@@ -1301,6 +1573,9 @@ int main(int argc, char **argv)
 	init_nls();
 #endif
 
+	/* Init color strings */
+	init_colors();
+
 	/* Get HZ */
 	get_HZ();
 
@@ -1364,7 +1639,7 @@ int main(int argc, char **argv)
 				 * MAX_NAME_LEN - 2: one char for the heading space,
 				 * and one for the trailing '\0'.
 				 */
-				sprintf(group_name, " %-.*s", MAX_NAME_LEN - 2, argv[opt++]);
+				snprintf(group_name, MAX_NAME_LEN, " %-.*s", MAX_NAME_LEN - 2, argv[opt++]);
 			}
 			else {
 				usage(argv[0]);
@@ -1397,6 +1672,17 @@ int main(int argc, char **argv)
 			}
 		}
 
+		else if (!strcmp(argv[opt], "-o")) {
+			/* Select output format */
+			if (argv[++opt] && !strcmp(argv[opt], K_JSON)) {
+				flags |= I_D_JSON_OUTPUT;
+				opt++;
+			}
+			else {
+				usage(argv[0]);
+			}
+		}
+
 #ifdef DEBUG
 		else if (!strcmp(argv[opt], "--debuginfo")) {
 			flags |= I_D_DEBUG;
@@ -1421,6 +1707,11 @@ int main(int argc, char **argv)
 					report_set = TRUE;
 					break;
 
+                                case 'H':
+					/* Display stats only for the groups */
+					flags |= I_D_GROUP_TOTAL_ONLY;
+					break;
+
 				case 'h':
 					/*
 					 * Display device utilization report
@@ -1455,11 +1746,6 @@ int main(int argc, char **argv)
 					flags |= I_D_PARTITIONS + I_D_PART_ALL;
 					break;
 
-				case 'T':
-					/* Display stats only for the groups */
-					flags |= I_D_GROUP_TOTAL_ONLY;
-					break;
-
 				case 't':
 					/* Display timestamp */
 					flags |= I_D_TIMESTAMP;
@@ -1495,7 +1781,7 @@ int main(int argc, char **argv)
 		else if (!isdigit(argv[opt][0])) {
 			/*
 			 * By default iostat doesn't display unused devices.
-			 * If some devices are explictly entered on the command line
+			 * If some devices are explicitly entered on the command line
 			 * then don't apply this rule any more.
 			 */
 			flags |= I_D_UNFILTERED;
@@ -1586,14 +1872,17 @@ int main(int argc, char **argv)
 	/* Get system name, release number and hostname */
 	uname(&header);
 	if (print_gal_header(&rectime, header.sysname, header.release,
-			     header.nodename, header.machine, cpu_nr)) {
+			     header.nodename, header.machine, cpu_nr,
+			     DISPLAY_JSON_OUTPUT(flags))) {
 		flags |= I_D_ISO;
 	}
-	printf("\n");
+	if (!DISPLAY_JSON_OUTPUT(flags)) {
+		printf("\n");
+	}
 
 	/* Set a handler for SIGALRM */
 	memset(&alrm_act, 0, sizeof(alrm_act));
-	alrm_act.sa_handler = (void *) alarm_handler;
+	alrm_act.sa_handler = alarm_handler;
 	sigaction(SIGALRM, &alrm_act, NULL);
 	alarm(interval);
 
