@@ -1,6 +1,6 @@
 /*
  * json_stats.c: Funtions used by sadf to display statistics in JSON format.
- * (C) 1999-2014 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1999-2016 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -15,7 +15,7 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc., *
- * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                   *
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA              *
  ***************************************************************************
  */
 
@@ -115,12 +115,7 @@ __print_funct_t json_print_cpu_stats(struct activity *a, int curr, int tab,
 	struct stats_cpu *scc, *scp;
 	char cpuno[8];
 
-	if (DISPLAY_CPU_DEF(a->opt_flags)) {
-		xprintf(tab++, "\"cpu-load\": [");
-	}
-	else if (DISPLAY_CPU_ALL(a->opt_flags)) {
-		xprintf(tab++, "\"cpu-load-all\": [");
-	}
+	xprintf(tab++, "\"cpu-load\": [");
 
 	for (i = 0; (i < a->nr) && (i < a->bitmap->b_size + 1); i++) {
 
@@ -128,136 +123,137 @@ __print_funct_t json_print_cpu_stats(struct activity *a, int curr, int tab,
 		scp = (struct stats_cpu *) ((char *) a->buf[!curr] + i * a->msize);
 
 		/* Should current CPU (including CPU "all") be displayed? */
-		if (a->bitmap->b_array[i >> 3] & (1 << (i & 0x07))) {
+		if (!(a->bitmap->b_array[i >> 3] & (1 << (i & 0x07))))
+			/* No */
+			continue;
 
-			/* Yes: Display current CPU stats */
+		/* Yes: Display current CPU stats */
 
-			if (sep) {
-				printf(",\n");
-			}
-			sep = TRUE;
+		if (sep) {
+			printf(",\n");
+		}
+		sep = TRUE;
 
-			if (!i) {
-				/* This is CPU "all" */
-				strcpy(cpuno, "all");
+		if (!i) {
+			/* This is CPU "all" */
+			strcpy(cpuno, "all");
+		}
+		else {
+			sprintf(cpuno, "%d", i - 1);
+
+			/*
+			 * If the CPU is offline then it is omited from /proc/stat:
+			 * All the fields couldn't have been read and the sum of them is zero.
+			 * (Remember that guest/guest_nice times are already included in
+			 * user/nice modes.)
+			 */
+			if ((scc->cpu_user    + scc->cpu_nice + scc->cpu_sys   +
+			     scc->cpu_iowait  + scc->cpu_idle + scc->cpu_steal +
+			     scc->cpu_hardirq + scc->cpu_softirq) == 0) {
+				/*
+				 * Set current struct fields (which have been set to zero)
+				 * to values from previous iteration. Hence their values won't
+				 * jump from zero when the CPU comes back online.
+				 */
+				*scc = *scp;
+
+				g_itv = 0;
+				cpu_offline = TRUE;
 			}
 			else {
-				sprintf(cpuno, "%d", i - 1);
-
 				/*
-				 * If the CPU is offline then it is omited from /proc/stat:
-				 * All the fields couldn't have been read and the sum of them is zero.
-				 * (Remember that guest/guest_nice times are already included in
-				 * user/nice modes.)
+				 * Recalculate interval for current proc.
+				 * If result is 0 then current CPU is a tickless one.
 				 */
-				if ((scc->cpu_user    + scc->cpu_nice + scc->cpu_sys   +
-				     scc->cpu_iowait  + scc->cpu_idle + scc->cpu_steal +
-				     scc->cpu_hardirq + scc->cpu_softirq) == 0) {
-					/*
-					 * Set current struct fields (which have been set to zero)
-					 * to values from previous iteration. Hence their values won't
-					 * jump from zero when the CPU comes back online.
-					 */
-					*scc = *scp;
-
-					g_itv = 0;
-					cpu_offline = TRUE;
-				}
-				else {
-					/*
-					 * Recalculate interval for current proc.
-					 * If result is 0 then current CPU is a tickless one.
-					 */
-					g_itv = get_per_cpu_interval(scc, scp);
-					cpu_offline = FALSE;
-				}
-
-				if (!g_itv) {
-					/* Current CPU is offline or tickless */
-					if (DISPLAY_CPU_DEF(a->opt_flags)) {
-						xprintf0(tab, "{\"cpu\": \"%d\", "
-							 "\"user\": %.2f, "
-							 "\"nice\": %.2f, "
-							 "\"system\": %.2f, "
-							 "\"iowait\": %.2f, "
-							 "\"steal\": %.2f, "
-							 "\"idle\": %.2f}",
-							 i - 1, 0.0, 0.0, 0.0, 0.0, 0.0,
-							 cpu_offline ? 0.0 : 100.0);
-					}
-					else if (DISPLAY_CPU_ALL(a->opt_flags)) {
-						xprintf0(tab, "{\"cpu\": \"%d\", "
-							 "\"usr\": %.2f, "
-							 "\"nice\": %.2f, "
-							 "\"sys\": %.2f, "
-							 "\"iowait\": %.2f, "
-							 "\"steal\": %.2f, "
-							 "\"irq\": %.2f, "
-							 "\"soft\": %.2f, "
-							 "\"guest\": %.2f, "
-							 "\"gnice\": %.2f, "
-							 "\"idle\": %.2f}",
-							 i - 1, 0.0, 0.0, 0.0, 0.0,
-							 0.0, 0.0, 0.0, 0.0, 0.0,
-							 cpu_offline ? 0.0 : 100.0);
-					}
-					continue;
-				}
+				g_itv = get_per_cpu_interval(scc, scp);
+				cpu_offline = FALSE;
 			}
 
-			if (DISPLAY_CPU_DEF(a->opt_flags)) {
-				xprintf0(tab, "{\"cpu\": \"%s\", "
-					 "\"user\": %.2f, "
-					 "\"nice\": %.2f, "
-					 "\"system\": %.2f, "
-					 "\"iowait\": %.2f, "
-					 "\"steal\": %.2f, "
-					 "\"idle\": %.2f}",
-					 cpuno,
-					 ll_sp_value(scp->cpu_user, scc->cpu_user, g_itv),
-					 ll_sp_value(scp->cpu_nice, scc->cpu_nice, g_itv),
-					 ll_sp_value(scp->cpu_sys + scp->cpu_hardirq + scp->cpu_softirq,
-						     scc->cpu_sys + scc->cpu_hardirq + scc->cpu_softirq,
-						     g_itv),
-					 ll_sp_value(scp->cpu_iowait, scc->cpu_iowait, g_itv),
-					 ll_sp_value(scp->cpu_steal, scc->cpu_steal, g_itv),
-					 scc->cpu_idle < scp->cpu_idle ?
-					 0.0 :
-					 ll_sp_value(scp->cpu_idle, scc->cpu_idle, g_itv));
-			}
-			else if (DISPLAY_CPU_ALL(a->opt_flags)) {
-				xprintf0(tab, "{\"cpu\": \"%s\", "
-					 "\"usr\": %.2f, "
-					 "\"nice\": %.2f, "
-					 "\"sys\": %.2f, "
-					 "\"iowait\": %.2f, "
-					 "\"steal\": %.2f, "
-					 "\"irq\": %.2f, "
-					 "\"soft\": %.2f, "
-					 "\"guest\": %.2f, "
-					 "\"gnice\": %.2f, "
-					 "\"idle\": %.2f}",
-					 cpuno,
-					 (scc->cpu_user - scc->cpu_guest) < (scp->cpu_user - scp->cpu_guest) ?
-					 0.0 :
-					 ll_sp_value(scp->cpu_user - scp->cpu_guest,
-						     scc->cpu_user - scc->cpu_guest, g_itv),
-					 (scc->cpu_nice - scc->cpu_guest_nice) < (scp->cpu_nice - scp->cpu_guest_nice) ?
-					 0.0 :
-					 ll_sp_value(scp->cpu_nice - scp->cpu_guest_nice,
-						     scc->cpu_nice - scc->cpu_guest_nice, g_itv),
-					 ll_sp_value(scp->cpu_sys, scc->cpu_sys, g_itv),
-					 ll_sp_value(scp->cpu_iowait, scc->cpu_iowait, g_itv),
-					 ll_sp_value(scp->cpu_steal, scc->cpu_steal, g_itv),
-					 ll_sp_value(scp->cpu_hardirq, scc->cpu_hardirq, g_itv),
-					 ll_sp_value(scp->cpu_softirq, scc->cpu_softirq, g_itv),
-					 ll_sp_value(scp->cpu_guest, scc->cpu_guest, g_itv),
-					 ll_sp_value(scp->cpu_guest_nice, scc->cpu_guest_nice, g_itv),
-					 scc->cpu_idle < scp->cpu_idle ?
-					 0.0 :
-					 ll_sp_value(scp->cpu_idle, scc->cpu_idle, g_itv));
+			if (!g_itv) {
+				/* Current CPU is offline or tickless */
+				if (DISPLAY_CPU_DEF(a->opt_flags)) {
+					xprintf0(tab, "{\"cpu\": \"%d\", "
+						 "\"user\": %.2f, "
+						 "\"nice\": %.2f, "
+						 "\"system\": %.2f, "
+						 "\"iowait\": %.2f, "
+						 "\"steal\": %.2f, "
+						 "\"idle\": %.2f}",
+						 i - 1, 0.0, 0.0, 0.0, 0.0, 0.0,
+						 cpu_offline ? 0.0 : 100.0);
+				}
+				else if (DISPLAY_CPU_ALL(a->opt_flags)) {
+					xprintf0(tab, "{\"cpu\": \"%d\", "
+						 "\"usr\": %.2f, "
+						 "\"nice\": %.2f, "
+						 "\"sys\": %.2f, "
+						 "\"iowait\": %.2f, "
+						 "\"steal\": %.2f, "
+						 "\"irq\": %.2f, "
+						 "\"soft\": %.2f, "
+						 "\"guest\": %.2f, "
+						 "\"gnice\": %.2f, "
+						 "\"idle\": %.2f}",
+						 i - 1, 0.0, 0.0, 0.0, 0.0,
+						 0.0, 0.0, 0.0, 0.0, 0.0,
+						 cpu_offline ? 0.0 : 100.0);
+				}
+				continue;
 			}
 		}
+
+		if (DISPLAY_CPU_DEF(a->opt_flags)) {
+			xprintf0(tab, "{\"cpu\": \"%s\", "
+				 "\"user\": %.2f, "
+				 "\"nice\": %.2f, "
+				 "\"system\": %.2f, "
+				 "\"iowait\": %.2f, "
+				 "\"steal\": %.2f, "
+				 "\"idle\": %.2f}",
+				 cpuno,
+				 ll_sp_value(scp->cpu_user, scc->cpu_user, g_itv),
+				 ll_sp_value(scp->cpu_nice, scc->cpu_nice, g_itv),
+				 ll_sp_value(scp->cpu_sys + scp->cpu_hardirq + scp->cpu_softirq,
+					     scc->cpu_sys + scc->cpu_hardirq + scc->cpu_softirq,
+					     g_itv),
+				 ll_sp_value(scp->cpu_iowait, scc->cpu_iowait, g_itv),
+				 ll_sp_value(scp->cpu_steal, scc->cpu_steal, g_itv),
+				 scc->cpu_idle < scp->cpu_idle ?
+				 0.0 :
+				 ll_sp_value(scp->cpu_idle, scc->cpu_idle, g_itv));
+		}
+		else if (DISPLAY_CPU_ALL(a->opt_flags)) {
+			xprintf0(tab, "{\"cpu\": \"%s\", "
+				 "\"usr\": %.2f, "
+				 "\"nice\": %.2f, "
+				 "\"sys\": %.2f, "
+				 "\"iowait\": %.2f, "
+				 "\"steal\": %.2f, "
+				 "\"irq\": %.2f, "
+				 "\"soft\": %.2f, "
+				 "\"guest\": %.2f, "
+				 "\"gnice\": %.2f, "
+				 "\"idle\": %.2f}",
+				 cpuno,
+				 (scc->cpu_user - scc->cpu_guest) < (scp->cpu_user - scp->cpu_guest) ?
+				 0.0 :
+				 ll_sp_value(scp->cpu_user - scp->cpu_guest,
+					     scc->cpu_user - scc->cpu_guest, g_itv),
+				 (scc->cpu_nice - scc->cpu_guest_nice) < (scp->cpu_nice - scp->cpu_guest_nice) ?
+				 0.0 :
+				 ll_sp_value(scp->cpu_nice - scp->cpu_guest_nice,
+					     scc->cpu_nice - scc->cpu_guest_nice, g_itv),
+				 ll_sp_value(scp->cpu_sys, scc->cpu_sys, g_itv),
+				 ll_sp_value(scp->cpu_iowait, scc->cpu_iowait, g_itv),
+				 ll_sp_value(scp->cpu_steal, scc->cpu_steal, g_itv),
+				 ll_sp_value(scp->cpu_hardirq, scc->cpu_hardirq, g_itv),
+				 ll_sp_value(scp->cpu_softirq, scc->cpu_softirq, g_itv),
+				 ll_sp_value(scp->cpu_guest, scc->cpu_guest, g_itv),
+				 ll_sp_value(scp->cpu_guest_nice, scc->cpu_guest_nice, g_itv),
+				 scc->cpu_idle < scp->cpu_idle ?
+				 0.0 :
+				 ll_sp_value(scp->cpu_idle, scc->cpu_idle, g_itv));
+		}
 	}
 
 	printf("\n");
@@ -287,7 +283,7 @@ __print_funct_t json_print_pcsw_stats(struct activity *a, int curr, int tab,
 		 "\"proc\": %.2f, "
 		 "\"cswch\": %.2f}",
 		 S_VALUE(spp->processes, spc->processes, itv),
-		 ll_s_value(spp->context_switch, spc->context_switch, itv));
+		 S_VALUE(spp->context_switch, spc->context_switch, itv));
 }
 
 /*
@@ -337,7 +333,7 @@ __print_funct_t json_print_irq_stats(struct activity *a, int curr, int tab,
 			xprintf0(tab, "{\"intr\": \"%s\", "
 				 "\"value\": %.2f}",
 				 irqno,
-				 ll_s_value(sip->irq_nr, sic->irq_nr, itv));
+				 S_VALUE(sip->irq_nr, sic->irq_nr, itv));
 		}
 	}
 
@@ -439,10 +435,21 @@ __print_funct_t json_print_io_stats(struct activity *a, int curr, int tab,
 		 "\"io-writes\": {"
 		 "\"wtps\": %.2f, "
 		 "\"bwrtn\": %.2f}}",
+		 /*
+		  * If we get negative values, this is probably because
+		  * one or more devices/filesystems have been unmounted.
+		  * We display 0.0 in this case though we should rather tell
+		  * the user that the value cannot be calculated here.
+		  */
+		 sic->dk_drive < sip->dk_drive ? 0.0 :
 		 S_VALUE(sip->dk_drive, sic->dk_drive, itv),
-		 S_VALUE(sip->dk_drive_rio,  sic->dk_drive_rio,  itv),
+		 sic->dk_drive_rio < sip->dk_drive_rio ? 0.0 :
+		 S_VALUE(sip->dk_drive_rio, sic->dk_drive_rio, itv),
+		 sic->dk_drive_rblk < sip->dk_drive_rblk ? 0.0 :
 		 S_VALUE(sip->dk_drive_rblk, sic->dk_drive_rblk, itv),
-		 S_VALUE(sip->dk_drive_wio,  sic->dk_drive_wio,  itv),
+		 sic->dk_drive_wio < sip->dk_drive_wio ? 0.0 :
+		 S_VALUE(sip->dk_drive_wio, sic->dk_drive_wio, itv),
+		 sic->dk_drive_wblk < sip->dk_drive_wblk ? 0.0 :
 		 S_VALUE(sip->dk_drive_wblk, sic->dk_drive_wblk, itv));
 }
 
@@ -495,6 +502,20 @@ __print_funct_t json_print_memory_stats(struct activity *a, int curr, int tab,
 		       smc->activekb,
 		       smc->inactkb,
 		       smc->dirtykb);
+
+		if (DISPLAY_MEM_ALL(a->opt_flags)) {
+			/* Display extended memory stats */
+			printf(", \"anonpg\": %lu, "
+			       "\"slab\": %lu, "
+			       "\"kstack\": %lu, "
+			       "\"pgtbl\": %lu, "
+			       "\"vmused\": %lu",
+			       smc->anonpgkb,
+			       smc->slabkb,
+			       smc->kstackkb,
+			       smc->pgtblkb,
+			       smc->vmusedkb);
+		}
 	}
 
 	if (DISPLAY_SWAP(a->opt_flags)) {
@@ -729,8 +750,8 @@ __print_funct_t json_print_disk_stats(struct activity *a, int curr, int tab,
 			 /* Confusion possible here between index and minor numbers */
 			 dev_name,
 			 S_VALUE(sdp->nr_ios, sdc->nr_ios, itv),
-			 ll_s_value(sdp->rd_sect, sdc->rd_sect, itv),
-			 ll_s_value(sdp->wr_sect, sdc->wr_sect, itv),
+			 S_VALUE(sdp->rd_sect, sdc->rd_sect, itv),
+			 S_VALUE(sdp->wr_sect, sdc->wr_sect, itv),
 			 /* See iostat for explanations */
 			 xds.arqsz,
 			 S_VALUE(sdp->rq_ticks, sdc->rq_ticks, itv) / 1000.0,
@@ -822,7 +843,7 @@ close_json_markup:
 
 /*
  ***************************************************************************
- * Display network interfaces error statistics in JSON.
+ * Display network interfaces errors statistics in JSON.
  *
  * IN:
  * @a		Activity structure with statistics.
@@ -1089,7 +1110,7 @@ close_json_markup:
 
 /*
  ***************************************************************************
- * Display IP network error statistics in JSON.
+ * Display IP network errors statistics in JSON.
  *
  * IN:
  * @a		Activity structure with statistics.
@@ -1199,7 +1220,7 @@ close_json_markup:
 
 /*
  ***************************************************************************
- * Display ICMP error message statistics in JSON.
+ * Display ICMP errors message statistics in JSON.
  *
  * IN:
  * @a		Activity structure with statistics.
@@ -1297,7 +1318,7 @@ close_json_markup:
 
 /*
  ***************************************************************************
- * Display TCP network error statistics in JSON.
+ * Display TCP network errors statistics in JSON.
  *
  * IN:
  * @a		Activity structure with statistics.
@@ -1474,7 +1495,7 @@ close_json_markup:
 
 /*
  ***************************************************************************
- * Display IPv6 network error statistics in JSON.
+ * Display IPv6 network errors statistics in JSON.
  *
  * IN:
  * @a		Activity structure with statistics.
@@ -1596,7 +1617,7 @@ close_json_markup:
 
 /*
  ***************************************************************************
- * Display ICMPv6 error message statistics in JSON.
+ * Display ICMPv6 error messages statistics in JSON.
  *
  * IN:
  * @a		Activity structure with statistics.
@@ -2117,7 +2138,7 @@ __print_funct_t json_print_filesystem_stats(struct activity *a, int curr, int ta
 		sfc = (struct stats_filesystem *) ((char *) a->buf[curr]  + i * a->msize);
 
 		if (!sfc->f_blocks)
-			/* Size of filesystem is null: We are at the end of the list */
+			/* Size of filesystem is zero: We are at the end of the list */
 			break;
 
 		if (sep) {
@@ -2125,7 +2146,7 @@ __print_funct_t json_print_filesystem_stats(struct activity *a, int curr, int ta
 		}
 		sep = TRUE;
 
-		xprintf0(tab, "{\"filesystem\": \"%s\", "
+		xprintf0(tab, "{\"%s\": \"%s\", "
 			 "\"MBfsfree\": %.0f, "
 			 "\"MBfsused\": %.0f, "
 			 "\"%%fsused\": %.2f, "
@@ -2133,7 +2154,8 @@ __print_funct_t json_print_filesystem_stats(struct activity *a, int curr, int ta
 			 "\"Ifree\": %llu, "
 			 "\"Iused\": %llu, "
 			 "\"%%Iused\": %.2f}",
-			 sfc->fs_name,
+			 DISPLAY_MOUNT(a->opt_flags) ? "mountpoint" : "filesystem",
+			 DISPLAY_MOUNT(a->opt_flags) ? sfc->mountp : sfc->fs_name,
 			 (double) sfc->f_bfree / 1024 / 1024,
 			 (double) (sfc->f_blocks - sfc->f_bfree) / 1024 / 1024,
 			 sfc->f_blocks ? SP_VALUE(sfc->f_bfree, sfc->f_blocks, sfc->f_blocks)
@@ -2149,3 +2171,65 @@ __print_funct_t json_print_filesystem_stats(struct activity *a, int curr, int ta
 	printf("\n");
 	xprintf0(--tab, "]");
 }
+
+/*
+ ***************************************************************************
+ * Display Fibre Channel HBA statistics in JSON.
+ *
+ * IN:
+ * @a		Activity structure with statistics.
+ * @curr	Index in array for current sample statistics.
+ * @tab		Indentation in output.
+ * @itv		Interval of time in jiffies.
+ ***************************************************************************
+ */
+__print_funct_t json_print_fchost_stats(struct activity *a, int curr, int tab,
+					unsigned long long itv)
+{
+	int i;
+	struct stats_fchost *sfcc, *sfcp;
+	int sep = FALSE;
+
+	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+		goto close_json_markup;
+
+	json_markup_network(tab, OPEN_JSON_MARKUP);
+	tab++;
+
+	xprintf(tab++, "\"fchosts\": [");
+
+	for (i = 0; i < a->nr; i++) {
+		sfcc = (struct stats_fchost *) ((char *) a->buf[curr]  + i * a->msize);
+		sfcp = (struct stats_fchost *) ((char *) a->buf[!curr]  + i * a->msize);
+
+		if (!sfcc->fchost_name[0])
+			/* We are at the end of the list */
+			break;
+
+		if (sep)
+			printf(",\n");
+
+		sep = TRUE;
+
+		xprintf0(tab, "{\"fchost\": \"%s\", "
+			 "\"fch_rxf\": %.2f, "
+			 "\"fch_txf\": %.2f, "
+			 "\"fch_rxw\": %.2f, "
+			 "\"fch_txw\": %.2f}",
+			 sfcc->fchost_name,
+			 S_VALUE(sfcp->f_rxframes, sfcc->f_rxframes, itv),
+			 S_VALUE(sfcp->f_txframes, sfcc->f_txframes, itv),
+			 S_VALUE(sfcp->f_rxwords,  sfcc->f_rxwords,  itv),
+			 S_VALUE(sfcp->f_txwords,  sfcc->f_txwords,  itv));
+	}
+
+	printf("\n");
+	xprintf0(--tab, "]");
+
+	tab --;
+
+close_json_markup:
+	if (CLOSE_MARKUP(a->options)) {
+		json_markup_network(tab, CLOSE_JSON_MARKUP);
+	}
+}
