@@ -1,6 +1,6 @@
 /*
  * rd_stats.c: Read system statistics
- * (C) 1999-2014 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1999-2016 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -15,7 +15,7 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc., *
- * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                   *
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA              *
  ***************************************************************************
  */
 
@@ -27,7 +27,7 @@
 #include <ctype.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <sys/vfs.h>
+#include <sys/statvfs.h>
 #include <unistd.h>
 
 #include "common.h"
@@ -268,6 +268,26 @@ void read_meminfo(struct stats_memory *st_memory)
 			/* Read the amount of commited memory in kB */
 			sscanf(line + 13, "%lu", &st_memory->comkb);
 		}
+		else if (!strncmp(line, "AnonPages:", 10)) {
+			/* Read the amount of pages mapped into userspace page tables in kB */
+			sscanf(line + 10, "%lu", &st_memory->anonpgkb);
+		}
+		else if (!strncmp(line, "Slab:", 5)) {
+			/* Read the amount of in-kernel data structures cache in kB */
+			sscanf(line + 5, "%lu", &st_memory->slabkb);
+		}
+		else if (!strncmp(line, "KernelStack:", 12)) {
+			/* Read the kernel stack utilization in kB */
+			sscanf(line + 12, "%lu", &st_memory->kstackkb);
+		}
+		else if (!strncmp(line, "PageTables:", 11)) {
+			/* Read the amount of memory dedicated to the lowest level of page tables in kB */
+			sscanf(line + 11, "%lu", &st_memory->pgtblkb);
+		}
+		else if (!strncmp(line, "VmallocUsed:", 12)) {
+			/* Read the amount of vmalloc area which is used in kB */
+			sscanf(line + 12, "%lu", &st_memory->vmusedkb);
+		}
 	}
 
 	fclose(fp);
@@ -394,20 +414,24 @@ void read_loadavg(struct stats_queue *st_queue)
 	FILE *fp;
 	char line[8192];
 	int load_tmp[3];
+	int rc;
 
 	if ((fp = fopen(LOADAVG, "r")) == NULL)
 		return;
 
 	/* Read load averages and queue length */
-	fscanf(fp, "%d.%d %d.%d %d.%d %ld/%d %*d\n",
-	       &load_tmp[0], &st_queue->load_avg_1,
-	       &load_tmp[1], &st_queue->load_avg_5,
-	       &load_tmp[2], &st_queue->load_avg_15,
-	       &st_queue->nr_running,
-	       &st_queue->nr_threads);
+	rc = fscanf(fp, "%d.%u %d.%u %d.%u %lu/%u %*d\n",
+		    &load_tmp[0], &st_queue->load_avg_1,
+		    &load_tmp[1], &st_queue->load_avg_5,
+		    &load_tmp[2], &st_queue->load_avg_15,
+		    &st_queue->nr_running,
+		    &st_queue->nr_threads);
 
 	fclose(fp);
 
+	if (rc < 8)
+		return;
+
 	st_queue->load_avg_1  += load_tmp[0] * 100;
 	st_queue->load_avg_5  += load_tmp[1] * 100;
 	st_queue->load_avg_15 += load_tmp[2] * 100;
@@ -663,7 +687,7 @@ void read_tty_driver_serial(struct stats_serial *st_serial, int nbr)
 			sscanf(line, "%u", &st_serial_i->line);
 			/*
 			 * A value of 0 means an unused structure.
-			 * So increment it to make sure it is not null.
+			 * So increment it to make sure it is not zero.
 			 */
 			(st_serial_i->line)++;
 			/*
@@ -709,43 +733,60 @@ void read_kernel_tables(struct stats_ktables *st_ktables)
 {
 	FILE *fp;
 	unsigned int parm;
+	int rc = 0;
 
 	/* Open /proc/sys/fs/dentry-state file */
 	if ((fp = fopen(FDENTRY_STATE, "r")) != NULL) {
-		fscanf(fp, "%*d %u",
-		       &st_ktables->dentry_stat);
+		rc = fscanf(fp, "%*d %u",
+			    &st_ktables->dentry_stat);
 		fclose(fp);
+		if (rc == 0) {
+			st_ktables->dentry_stat = 0;
+		}
 	}
 
 	/* Open /proc/sys/fs/file-nr file */
 	if ((fp = fopen(FFILE_NR, "r")) != NULL) {
-		fscanf(fp, "%u %u",
-		       &st_ktables->file_used, &parm);
+		rc = fscanf(fp, "%u %u",
+			    &st_ktables->file_used, &parm);
 		fclose(fp);
 		/*
 		 * The number of used handles is the number of allocated ones
 		 * minus the number of free ones.
 		 */
-		st_ktables->file_used -= parm;
+		if (rc == 2) {
+			st_ktables->file_used -= parm;
+		}
+		else {
+			st_ktables->file_used = 0;
+		}
 	}
 
 	/* Open /proc/sys/fs/inode-state file */
 	if ((fp = fopen(FINODE_STATE, "r")) != NULL) {
-		fscanf(fp, "%u %u",
-		       &st_ktables->inode_used, &parm);
+		rc = fscanf(fp, "%u %u",
+			    &st_ktables->inode_used, &parm);
 		fclose(fp);
 		/*
 		 * The number of inuse inodes is the number of allocated ones
 		 * minus the number of free ones.
 		 */
-		st_ktables->inode_used -= parm;
+		if (rc == 2) {
+			st_ktables->inode_used -= parm;
+		}
+		else {
+			st_ktables->inode_used = 0;
+		}
 	}
 
 	/* Open /proc/sys/kernel/pty/nr file */
 	if ((fp = fopen(PTY_NR, "r")) != NULL) {
-		fscanf(fp, "%u",
-		       &st_ktables->pty_nr);
+		rc = fscanf(fp, "%u",
+			    &st_ktables->pty_nr);
 		fclose(fp);
+		if (rc == 0) {
+			st_ktables->pty_nr = 0;
+		}
 	}
 }
 
@@ -856,9 +897,13 @@ void read_if_info(struct stats_net_dev *st_net_dev, int nbr)
 			/* Cannot read NIC speed */
 			continue;
 
-		fscanf(fp, "%u", &st_net_dev_i->speed);
+		n = fscanf(fp, "%u", &st_net_dev_i->speed);
 
 		fclose(fp);
+
+		if (n != 1) {
+			st_net_dev_i->speed = 0;
+		}
 	}
 }
 
@@ -1127,7 +1172,7 @@ void read_net_ip(struct stats_net_ip *st_net_ip)
 
 /*
  ***************************************************************************
- * Read IP network error statistics from /proc/net/snmp.
+ * Read IP network errors statistics from /proc/net/snmp.
  *
  * IN:
  * @st_net_eip	Structure where stats will be saved.
@@ -1186,6 +1231,7 @@ void read_net_icmp(struct stats_net_icmp *st_net_icmp)
 {
 	FILE *fp;
 	char line[1024];
+	static char format[256] = "";
 	int sw = FALSE;
 
 	if ((fp = fopen(NET_SNMP, "r")) == NULL)
@@ -1195,9 +1241,7 @@ void read_net_icmp(struct stats_net_icmp *st_net_icmp)
 
 		if (!strncmp(line, "Icmp:", 5)) {
 			if (sw) {
-				sscanf(line + 5, "%lu %*u %*u %*u %*u %*u %*u "
-				       "%lu %lu %lu %lu %lu %lu %lu %*u %*u %*u %*u "
-				       "%*u %*u %lu %lu %lu %lu %lu %lu",
+				sscanf(line + 5, format,
 				       &st_net_icmp->InMsgs,
 				       &st_net_icmp->InEchos,
 				       &st_net_icmp->InEchoReps,
@@ -1216,6 +1260,26 @@ void read_net_icmp(struct stats_net_icmp *st_net_icmp)
 				break;
 			}
 			else {
+				if (!strlen(format)) {
+					if (strstr(line, "InCsumErrors")) {
+						/*
+						 * New format: InCsumErrors field exists at position #3.
+						 * Capture: 1,9,10,11,12,13,14,15,22,23,24,25,26,27.
+						 */
+						strcpy(format, "%lu %*u %*u %*u %*u %*u %*u %*u "
+							       "%lu %lu %lu %lu %lu %lu %lu %*u %*u %*u %*u "
+							       "%*u %*u %lu %lu %lu %lu %lu %lu");
+					}
+					else {
+						/*
+						 * Old format: InCsumErrors field doesn't exist.
+						 * Capture: 1,8,9,10,11,12,13,14,21,22,23,24,25,26.
+						 */
+						strcpy(format, "%lu %*u %*u %*u %*u %*u %*u "
+							       "%lu %lu %lu %lu %lu %lu %lu %*u %*u %*u %*u "
+							       "%*u %*u %lu %lu %lu %lu %lu %lu");
+					}
+				}
 				sw = TRUE;
 			}
 		}
@@ -1226,7 +1290,7 @@ void read_net_icmp(struct stats_net_icmp *st_net_icmp)
 
 /*
  ***************************************************************************
- * Read ICMP network error statistics from /proc/net/snmp.
+ * Read ICMP network errors statistics from /proc/net/snmp.
  *
  * IN:
  * @st_net_eicmp	Structure where stats will be saved.
@@ -1318,7 +1382,7 @@ void read_net_tcp(struct stats_net_tcp *st_net_tcp)
 
 /*
  ***************************************************************************
- * Read TCP network error statistics from /proc/net/snmp.
+ * Read TCP network errors statistics from /proc/net/snmp.
  *
  * IN:
  * @st_net_etcp	Structure where stats will be saved.
@@ -1500,7 +1564,7 @@ void read_net_ip6(struct stats_net_ip6 *st_net_ip6)
 
 /*
  ***************************************************************************
- * Read IPv6 network error statistics from /proc/net/snmp6.
+ * Read IPv6 network errors statistics from /proc/net/snmp6.
  *
  * IN:
  * @st_net_eip6	Structure where stats will be saved.
@@ -1636,7 +1700,7 @@ void read_net_icmp6(struct stats_net_icmp6 *st_net_icmp6)
 
 /*
  ***************************************************************************
- * Read ICMPv6 network error statistics from /proc/net/snmp6.
+ * Read ICMPv6 network errors statistics from /proc/net/snmp6.
  *
  * IN:
  * @st_net_eicmp6	Structure where stats will be saved.
@@ -1896,8 +1960,9 @@ void read_time_in_state(struct stats_pwr_wghfreq *st_pwr_wghfreq, int cpu_nr, in
  */
 void read_usb_stats(struct stats_pwr_usb *st_pwr_usb, char *usb_device)
 {
-	int l;
+	int l, rc;
 	FILE *fp;
+	char * rs;
 	char filename[MAX_PF_NAME];
 
 	/* Get USB device bus number */
@@ -1907,36 +1972,47 @@ void read_usb_stats(struct stats_pwr_usb *st_pwr_usb, char *usb_device)
 	snprintf(filename, MAX_PF_NAME, "%s/%s/%s",
 		 SYSFS_USBDEV, usb_device, SYSFS_IDVENDOR);
 	if ((fp = fopen(filename, "r")) != NULL) {
-		fscanf(fp, "%x",
-		       &st_pwr_usb->vendor_id);
+		rc = fscanf(fp, "%x",
+			    &st_pwr_usb->vendor_id);
 		fclose(fp);
+		if (rc == 0) {
+			st_pwr_usb->vendor_id = 0;
+		}
 	}
 
 	/* Read USB device product ID */
 	snprintf(filename, MAX_PF_NAME, "%s/%s/%s",
 		 SYSFS_USBDEV, usb_device, SYSFS_IDPRODUCT);
 	if ((fp = fopen(filename, "r")) != NULL) {
-		fscanf(fp, "%x",
-		       &st_pwr_usb->product_id);
+		rc = fscanf(fp, "%x",
+			    &st_pwr_usb->product_id);
 		fclose(fp);
+		if (rc == 0) {
+			st_pwr_usb->product_id = 0;
+		}
 	}
 
 	/* Read USB device max power consumption */
 	snprintf(filename, MAX_PF_NAME, "%s/%s/%s",
 		 SYSFS_USBDEV, usb_device, SYSFS_BMAXPOWER);
 	if ((fp = fopen(filename, "r")) != NULL) {
-		fscanf(fp, "%u",
-		       &st_pwr_usb->bmaxpower);
+		rc = fscanf(fp, "%u",
+			    &st_pwr_usb->bmaxpower);
 		fclose(fp);
+		if (rc == 0) {
+			st_pwr_usb->bmaxpower = 0;
+		}
 	}
 
 	/* Read USB device manufacturer */
 	snprintf(filename, MAX_PF_NAME, "%s/%s/%s",
 		 SYSFS_USBDEV, usb_device, SYSFS_MANUFACTURER);
 	if ((fp = fopen(filename, "r")) != NULL) {
-		fgets(st_pwr_usb->manufacturer, MAX_MANUF_LEN - 1, fp);
+		rs = fgets(st_pwr_usb->manufacturer,
+			   MAX_MANUF_LEN - 1, fp);
 		fclose(fp);
-		if ((l = strlen(st_pwr_usb->manufacturer)) > 0) {
+		if ((rs != NULL) &&
+		    (l = strlen(st_pwr_usb->manufacturer)) > 0) {
 			/* Remove trailing CR */
 			st_pwr_usb->manufacturer[l - 1] = '\0';
 		}
@@ -1946,9 +2022,11 @@ void read_usb_stats(struct stats_pwr_usb *st_pwr_usb, char *usb_device)
 	snprintf(filename, MAX_PF_NAME, "%s/%s/%s",
 		 SYSFS_USBDEV, usb_device, SYSFS_PRODUCT);
 	if ((fp = fopen(filename, "r")) != NULL) {
-		fgets(st_pwr_usb->product, MAX_PROD_LEN - 1, fp);
+		rs = fgets(st_pwr_usb->product,
+			   MAX_PROD_LEN - 1, fp);
 		fclose(fp);
-		if ((l = strlen(st_pwr_usb->product)) > 0) {
+		if ((rs != NULL) &&
+		    (l = strlen(st_pwr_usb->product)) > 0) {
 			/* Remove trailing CR */
 			st_pwr_usb->product[l - 1] = '\0';
 		}
@@ -2012,10 +2090,10 @@ void read_bus_usb_dev(struct stats_pwr_usb *st_pwr_usb, int nbr)
 void read_filesystem(struct stats_filesystem *st_filesystem, int nbr)
 {
 	FILE *fp;
-	char line[256], fs_name[MAX_FS_LEN], mountp[128];
+	char line[512], fs_name[128], mountp[256];
 	int fs = 0;
 	struct stats_filesystem *st_filesystem_i;
-	struct statfs buf;
+	struct statvfs buf;
 
 	if ((fp = fopen(MTAB, "r")) == NULL)
 		return;
@@ -2023,27 +2101,129 @@ void read_filesystem(struct stats_filesystem *st_filesystem, int nbr)
 	while ((fgets(line, sizeof(line), fp) != NULL) && (fs < nbr)) {
 		if (line[0] == '/') {
 
-			/* Read current filesystem name and mount point */
-			sscanf(line, "%71s %127s", fs_name, mountp);
+			/* Read current filesystem name */
+			sscanf(line, "%127s", fs_name);
+			/*
+			 * And now read the corresponding mount point.
+			 * Read fs name and mount point in two distinct operations.
+			 * Indeed, if fs name length is greater than 127 chars,
+			 * previous scanf() will read only the first 127 chars, and
+			 * mount point name will be read using the remaining chars
+			 * from the fs name. This will result in a bogus name
+			 * and following statvfs() function will always fail.
+			 */
+			sscanf(strchr(line, ' ') + 1, "%255s", mountp);
 
 			/* Replace octal codes */
 			oct2chr(mountp);
 
-			if ((statfs(mountp, &buf) < 0) || (!buf.f_blocks))
+			/*
+			 * It's important to have read the whole mount point name
+			 * for statvfs() to work properly (see above).
+			 */
+			if ((statvfs(mountp, &buf) < 0) || (!buf.f_blocks))
 				continue;
 
 			st_filesystem_i = st_filesystem + fs++;
-			st_filesystem_i->f_blocks = buf.f_blocks * buf.f_bsize;
-			st_filesystem_i->f_bfree  = buf.f_bfree * buf.f_bsize;
-			st_filesystem_i->f_bavail = buf.f_bavail * buf.f_bsize;
+			st_filesystem_i->f_blocks = buf.f_blocks * buf.f_frsize;
+			st_filesystem_i->f_bfree  = buf.f_bfree * buf.f_frsize;
+			st_filesystem_i->f_bavail = buf.f_bavail * buf.f_frsize;
 			st_filesystem_i->f_files  = buf.f_files;
 			st_filesystem_i->f_ffree  = buf.f_ffree;
-			strcpy(st_filesystem_i->fs_name, fs_name);
+			strncpy(st_filesystem_i->fs_name, fs_name, MAX_FS_LEN);
+			st_filesystem_i->fs_name[MAX_FS_LEN - 1] = '\0';
+			strncpy(st_filesystem_i->mountp, mountp, MAX_FS_LEN);
+			st_filesystem_i->mountp[MAX_FS_LEN - 1] = '\0';
 		}
 	}
 
 	fclose(fp);
 }
 
+/*
+ ***************************************************************************
+ * Read Fibre Channel HBA statistics.
+ *
+ * IN:
+ * @st_fc	Structure where stats will be saved.
+ * @nbr		Total number of HBAs.
+ *
+ * OUT:
+ * @st_fc	Structure with statistics.
+ ***************************************************************************
+ */
+void read_fchost(struct stats_fchost *st_fc, int nbr)
+{
+	DIR *dir;
+	FILE *fp;
+	struct dirent *drd;
+	struct stats_fchost *st_fc_i;
+	int fch = 0;
+	char fcstat_filename[MAX_PF_NAME];
+	char line[256];
+	unsigned long rx_frames, tx_frames, rx_words, tx_words;
+
+	/* Each host, if present, will have its own hostX entry within SYSFS_FCHOST */
+	if ((dir = opendir(SYSFS_FCHOST)) == NULL)
+		return; /* No FC hosts */
+
+	/*
+	 * Read each of the counters via sysfs, where they are
+	 * returned as hex values (e.g. 0x72400).
+	 */
+	while (((drd = readdir(dir)) != NULL) && (fch < nbr)) {
+		rx_frames = tx_frames = rx_words = tx_words = 0;
+
+		if (!strncmp(drd->d_name, "host", 4)) {
+
+			snprintf(fcstat_filename, MAX_PF_NAME, FC_RX_FRAMES,
+				 SYSFS_FCHOST, drd->d_name);
+			if ((fp = fopen(fcstat_filename, "r"))) {
+				if (fgets(line, sizeof(line), fp)) {
+					sscanf(line, "%lx", &rx_frames);
+				}
+				fclose(fp);
+			}
+
+			snprintf(fcstat_filename, MAX_PF_NAME, FC_TX_FRAMES,
+				 SYSFS_FCHOST, drd->d_name);
+			if ((fp = fopen(fcstat_filename, "r"))) {
+				if (fgets(line, sizeof(line), fp)) {
+					sscanf(line, "%lx", &tx_frames);
+				}
+				fclose(fp);
+			}
+
+			snprintf(fcstat_filename, MAX_PF_NAME, FC_RX_WORDS,
+				 SYSFS_FCHOST, drd->d_name);
+			if ((fp = fopen(fcstat_filename, "r"))) {
+				if (fgets(line, sizeof(line), fp)) {
+					sscanf(line, "%lx", &rx_words);
+				}
+				fclose(fp);
+			}
+
+			snprintf(fcstat_filename, MAX_PF_NAME, FC_TX_WORDS,
+				 SYSFS_FCHOST, drd->d_name);
+			if ((fp = fopen(fcstat_filename, "r"))) {
+				if (fgets(line, sizeof(line), fp)) {
+					sscanf(line, "%lx", &tx_words);
+				}
+				fclose(fp);
+			}
+
+			st_fc_i = st_fc + fch++;
+			st_fc_i->f_rxframes = rx_frames;
+			st_fc_i->f_txframes = tx_frames;
+			st_fc_i->f_rxwords  = rx_words;
+			st_fc_i->f_txwords  = tx_words;
+			strncpy(st_fc_i->fchost_name, drd->d_name, MAX_FCH_LEN);
+			st_fc_i->fchost_name[MAX_FCH_LEN - 1] = '\0';
+		}
+
+	}
+	closedir(dir);
+}
+
 /*------------------ END: FUNCTIONS USED BY SADC ONLY ---------------------*/
 #endif /* SOURCE_SADC */
